@@ -5,30 +5,29 @@
 use std::cell::{OnceCell, RefCell};
 use std::ffi::CString;
 use std::os::raw::c_void;
-use std::ptr;
 
 use ews::{Mailbox, Recipient};
 use thin_vec::ThinVec;
 
 use cstr::cstr;
-use nserror::nsresult;
-use nserror::NS_OK;
+use nserror::{nsresult, NS_OK};
 use nsstring::{nsACString, nsCString, nsString};
 use url::Url;
 use uuid::Uuid;
-use xpcom::interfaces::{nsILoginInfo, nsILoginManager, nsIMsgOutgoingServer};
-use xpcom::{get_service, getter_addrefs, nsIID};
 use xpcom::{
+    get_service, getter_addrefs,
     interfaces::{
-        msgIAddressObject, nsIFile, nsIIOService, nsIMsgIdentity, nsIMsgOutgoingListener,
-        nsIMsgStatusFeedback, nsIMsgWindow, nsIPrefBranch, nsIPrefService, nsIURI, nsIUrlListener,
-        nsMsgAuthMethodValue, nsMsgSocketType, nsMsgSocketTypeValue,
+        msgIAddressObject, nsIFile, nsILoginInfo, nsILoginManager, nsIMsgIdentity,
+        nsIMsgOutgoingListener, nsIMsgOutgoingServer, nsIMsgStatusFeedback, nsIMsgWindow,
+        nsIPrefBranch, nsIPrefService, nsIURI, nsIUrlListener, nsMsgAuthMethodValue,
+        nsMsgSocketType, nsMsgSocketTypeValue,
     },
-    xpcom_method, RefPtr,
+    nsIID, xpcom_method, RefPtr,
 };
 
 use crate::authentication::credentials::AuthenticationProvider;
 use crate::client::XpComEwsClient;
+use crate::safe_xpcom::{SafeMsgOutgoingListener, SafeUri};
 use crate::xpcom_io;
 
 /// Whether a field is required to have a value (either in memory or in a pref)
@@ -550,14 +549,12 @@ impl EwsOutgoingServer {
     // Server URI
     xpcom_method!(server_uri => GetServerURI() -> *const nsIURI);
     fn server_uri(&self) -> Result<RefPtr<nsIURI>, nsresult> {
+        self.safe_server_uri().map(|uri| uri.into())
+    }
+
+    fn safe_server_uri(&self) -> Result<SafeUri, nsresult> {
         let url = self.ews_url()?;
-        let url = nsCString::from(url.as_str());
-
-        let io_service =
-            xpcom::get_service::<nsIIOService>(cstr!("@mozilla.org/network/io-service;1"))
-                .ok_or(nserror::NS_ERROR_FAILURE)?;
-
-        getter_addrefs(|p| unsafe { io_service.NewURI(&*url, ptr::null(), ptr::null(), p) })
+        SafeUri::new(url.as_str())
     }
 
     // Maximum number of connections
@@ -635,7 +632,7 @@ impl EwsOutgoingServer {
 
                 let mailbox = Mailbox {
                     name,
-                    email_address: address.to_string(),
+                    email_address: Some(address.to_string()),
                     ..Default::default()
                 };
 
@@ -649,10 +646,7 @@ impl EwsOutgoingServer {
             .query_interface::<nsIMsgOutgoingServer>()
             .ok_or(nserror::NS_ERROR_UNEXPECTED)?;
 
-        // TODO https://bugzilla.mozilla.org/show_bug.cgi?id=1987797 Accept
-        // override values for OAuth issuer details and pass override details
-        // into get_credentials.
-        let credentials = outgoing_server.get_credentials(None)?;
+        let credentials = outgoing_server.get_credentials()?;
 
         // Set up the client to build and send the request.
         let client = XpComEwsClient::new(url, outgoing_server, credentials)?;
@@ -665,8 +659,8 @@ impl EwsOutgoingServer {
                 message_id.to_utf8().into(),
                 should_request_dsn,
                 bcc_recipients,
-                RefPtr::new(listener),
-                self.server_uri()?,
+                SafeMsgOutgoingListener::new(listener),
+                self.safe_server_uri()?,
             ),
         )
         .detach();
