@@ -123,7 +123,8 @@ NS_IMPL_ISUPPORTS(Http3WebTransportStream, nsIInputStreamCallback,
                   nsIOutputStreamCallback)
 
 Http3WebTransportStream::Http3WebTransportStream(
-    Http3Session* aSession, uint64_t aSessionId, WebTransportStreamType aType,
+    Http3SessionBase* aSession, uint64_t aSessionId,
+    WebTransportStreamType aType,
     std::function<void(Result<RefPtr<WebTransportStreamBase>, nsresult>&&)>&&
         aCallback)
     : WebTransportStreamBase(aSessionId, std::move(aCallback)),
@@ -133,7 +134,7 @@ Http3WebTransportStream::Http3WebTransportStream(
   mStreamType = aType;
 }
 
-Http3WebTransportStream::Http3WebTransportStream(Http3Session* aSession,
+Http3WebTransportStream::Http3WebTransportStream(Http3SessionBase* aSession,
                                                  uint64_t aSessionId,
                                                  WebTransportStreamType aType,
                                                  uint64_t aStreamId)
