@@ -109,11 +109,20 @@ void nsHttpConnectionInfo::Init(const nsACString& host, int32_t port,
   mUsingHttpProxy = mUsingHttpsProxy || (proxyInfo && proxyInfo->IsHTTP());
 
   if (mUsingHttpProxy) {
-    mUsingConnect = mEndToEndSSL;  // SSL always uses CONNECT
-    uint32_t resolveFlags = 0;
-    if (NS_SUCCEEDED(mProxyInfo->GetResolveFlags(&resolveFlags)) &&
-        resolveFlags & nsIProtocolProxyService::RESOLVE_ALWAYS_TUNNEL) {
-      mUsingConnect = true;
+    mUsingConnect = mEndToEndSSL || proxyInfo->IsHttp3Proxy();
+    if (!mUsingConnect) {
+      uint32_t resolveFlags = 0;
+      if (NS_SUCCEEDED(mProxyInfo->GetResolveFlags(&resolveFlags)) &&
+          resolveFlags & nsIProtocolProxyService::RESOLVE_ALWAYS_TUNNEL) {
+        mUsingConnect = true;
+      }
+    }
+  }
+
+  if (mUsingHttpsProxy) {
+    mIsHttp3ProxyConnection = "masque"_ns.Equals(proxyInfo->Type());
+    if (mIsHttp3ProxyConnection) {
+      mProxyNPNToken = "h3"_ns;
     }
   }
 
@@ -486,13 +495,15 @@ nsHttpConnectionInfo::DeserializeHttpConnectionInfoCloneArgs(
   return cinfo.forget();
 }
 
-void nsHttpConnectionInfo::CloneAsDirectRoute(nsHttpConnectionInfo** outCI) {
+void nsHttpConnectionInfo::CloneAsDirectRoute(nsHttpConnectionInfo** outCI,
+                                              nsProxyInfo* aProxyInfo) {
   // Explicitly use an empty npnToken when |mIsHttp3| is true, since we want to
   // create a non-http3 connection info.
   RefPtr<nsHttpConnectionInfo> clone = new nsHttpConnectionInfo(
       mOrigin, mOriginPort,
       (mRoutedHost.IsEmpty() && !mIsHttp3) ? mNPNToken : ""_ns, mUsername,
-      mProxyInfo, mOriginAttributes, mEndToEndSSL, false, mWebTransport);
+      aProxyInfo ? aProxyInfo : mProxyInfo.get(), mOriginAttributes,
+      mEndToEndSSL, false, mWebTransport);
   // Make sure the anonymous, insecure-scheme, and private flags are transferred
   clone->SetAnonymous(GetAnonymous());
   clone->SetPrivate(GetPrivate());
@@ -512,6 +523,18 @@ void nsHttpConnectionInfo::CloneAsDirectRoute(nsHttpConnectionInfo** outCI) {
   clone.forget(outCI);
 }
 
+already_AddRefed<nsHttpConnectionInfo>
+nsHttpConnectionInfo::CreateConnectUDPFallbackConnInfo() {
+  if (!mProxyInfo || !mProxyInfo->IsHttp3Proxy()) {
+    return nullptr;
+  }
+
+  RefPtr<nsProxyInfo> proxyInfo = mProxyInfo->CreateFallbackProxyInfo();
+  RefPtr<nsHttpConnectionInfo> clone;
+  CloneAsDirectRoute(getter_AddRefs(clone), proxyInfo);
+  return clone.forget();
+}
+
 nsresult nsHttpConnectionInfo::CreateWildCard(nsHttpConnectionInfo** outParam) {
   // T???mozilla.org:443 (https:proxy.ducksong.com:3128) [specifc form]
   // TS??*:0 (https:proxy.ducksong.com:3128)   [wildcard form]
@@ -528,6 +551,7 @@ nsresult nsHttpConnectionInfo::CreateWildCard(nsHttpConnectionInfo** outParam) {
   // Make sure the anonymous and private flags are transferred!
   clone->SetAnonymous(GetAnonymous());
   clone->SetPrivate(GetPrivate());
+  clone->SetFallbackConnection(GetFallbackConnection());
   clone.forget(outParam);
   return NS_OK;
 }
