@@ -1,783 +1,785 @@
 /// Unique IDs for each statically-registered module.
 #[repr(u16)]
 pub enum ModuleID {
-  Anonymous042,
-  Anonymous054,
-  Console,
-  Anonymous199,
-  Anonymous509,
-  Anonymous409,
-  Anonymous171,
-  Anonymous034,
-  Anonymous339,
-  Anonymous478,
-  Predictor,
-  Anonymous550,
-  Anonymous312,
-  Anonymous605,
-  Anonymous207,
-  Anonymous029,
-  Anonymous282,
-  Anonymous237,
-  Anonymous585,
-  Anonymous213,
-  Anonymous344,
-  Anonymous270,
-  Anonymous489,
-  Anonymous188,
-  Anonymous313,
-  Anonymous536,
-  Anonymous047,
-  Anonymous330,
-  Prefetch,
-  Anonymous240,
-  Anonymous661,
-  Anonymous294,
-  ServiceWorkerManager,
-  Anonymous028,
-  Anonymous594,
-  Anonymous101,
-  RFPService,
-  Anonymous150,
-  AppStartup,
-  Anonymous170,
-  Anonymous480,
-  Anonymous611,
-  Anonymous645,
-  Anonymous090,
-  Anonymous165,
-  Anonymous406,
-  Anonymous022,
-  Anonymous387,
-  Anonymous506,
-  Anonymous420,
-  AddonContentPolicy,
-  Anonymous629,
-  GfxInfo,
-  Anonymous334,
-  Anonymous462,
-  Anonymous393,
-  Anonymous092,
-  Anonymous278,
-  Anonymous417,
-  Anonymous122,
-  IDN,
-  Anonymous127,
-  Anonymous052,
-  Locale,
-  Anonymous418,
-  Anonymous160,
-  Anonymous291,
-  Anonymous163,
-  StreamTransport,
-  Anonymous206,
-  Anonymous522,
-  Anonymous268,
-  Anonymous500,
-  Anonymous072,
-  Anonymous372,
-  Anonymous684,
-  Anonymous323,
-  Anonymous277,
-  Anonymous005,
-  Anonymous049,
-  Anonymous128,
-  Anonymous075,
-  Anonymous413,
-  Anonymous035,
-  URIFixup,
-  Anonymous215,
-  Anonymous535,
-  Anonymous665,
-  Anonymous421,
-  Anonymous517,
-  Anonymous086,
-  Anonymous469,
-  Anonymous572,
-  Anonymous261,
-  Anonymous004,
-  Anonymous608,
-  Anonymous300,
-  Anonymous216,
-  Anonymous316,
-  ProtocolProxy,
-  Anonymous373,
-  Copy,
-  Anonymous164,
-  Anonymous001,
-  Anonymous602,
-  Anonymous360,
-  Anonymous027,
-  XULRuntime,
-  Anonymous368,
-  Anonymous220,
-  Anonymous359,
-  Anonymous432,
-  Anonymous331,
-  Anonymous676,
-  Anonymous308,
-  Anonymous423,
-  RandomGenerator,
-  Anonymous100,
-  Anonymous415,
-  Anonymous063,
-  Anonymous023,
-  Anonymous525,
-  Anonymous527,
-  Anonymous530,
-  Anonymous107,
-  Anonymous293,
-  Anonymous419,
-  Anonymous484,
-  Anonymous450,
-  Anonymous512,
-  AuthPromptAdapter,
-  Anonymous381,
-  Anonymous448,
-  CredentialChooserService,
-  Anonymous520,
-  Anonymous601,
-  Anonymous634,
-  Anonymous132,
-  Anonymous178,
-  UrlClassifierStreamUpdater,
-  Anonymous342,
-  Anonymous328,
-  Anonymous498,
-  Anonymous014,
-  Anonymous071,
-  Anonymous453,
-  Anonymous551,
-  Anonymous606,
-  Anonymous076,
-  Anonymous581,
-  Anonymous157,
-  Anonymous655,
-  Anonymous241,
-  Anonymous324,
-  Anonymous562,
-  Anonymous374,
-  Anonymous232,
-  Anonymous320,
-  DBusHandlerApp,
-  Anonymous424,
-  Anonymous590,
-  Anonymous396,
-  Anonymous367,
-  Anonymous592,
-  Directory,
-  Anonymous589,
-  CaptivePortal,
-  PlacesPreviewsHelper,
-  Anonymous582,
-  Anonymous375,
-  Anonymous538,
-  Anonymous285,
-  Anonymous321,
-  Anonymous348,
-  Anonymous286,
-  Anonymous087,
-  Anonymous146,
-  Anonymous404,
-  Anonymous394,
-  Anonymous068,
-  Anonymous302,
-  Anonymous481,
-  Anonymous672,
-  Anonymous549,
-  ScriptSecurityManager,
-  Import,
-  Anonymous226,
-  Anonymous515,
-  Anonymous265,
-  PermissionManager,
-  Anonymous167,
-  Anonymous304,
-  Anonymous357,
-  Anonymous229,
-  Anonymous380,
-  Anonymous084,
-  Anonymous247,
-  Anonymous354,
-  Anonymous142,
-  Anonymous610,
-  Anonymous505,
-  Anonymous431,
-  Anonymous096,
-  Anonymous625,
-  FileProtocolHandler,
-  Anonymous474,
-  Anonymous683,
-  Anonymous327,
-  Anonymous569,
-  Anonymous362,
-  Anonymous297,
-  Anonymous082,
-  Anonymous061,
-  Anonymous016,
-  Anonymous258,
-  Anonymous192,
-  Anonymous228,
-  Anonymous311,
-  Anonymous214,
-  Anonymous173,
-  Anonymous471,
-  Anonymous410,
-  Anonymous566,
-  Anonymous526,
-  Anonymous521,
-  Anonymous256,
-  Anonymous059,
-  Folder,
-  Anonymous031,
-  Anonymous533,
-  Anonymous292,
-  Anonymous439,
-  Anonymous508,
-  IO,
-  Anonymous326,
-  BounceTrackingProtection,
-  Anonymous604,
-  Anonymous662,
-  Anonymous238,
-  Anonymous370,
-  Anonymous529,
-  Anonymous048,
-  Anonymous556,
-  Anonymous156,
-  Anonymous062,
-  StreamConverter,
-  Anonymous395,
-  Anonymous488,
-  Anonymous025,
-  Anonymous133,
-  Anonymous245,
-  Anonymous274,
-  Anonymous619,
-  Anonymous306,
-  Anonymous659,
-  Anonymous614,
-  MimeHeaderParam,
-  Anonymous037,
-  Anonymous032,
-  Anonymous584,
-  ChromeRegistry,
-  Anonymous670,
-  Anonymous123,
-  Anonymous537,
-  Anonymous586,
-  Anonymous222,
-  WindowWatcher,
-  Anonymous272,
-  Anonymous260,
-  Anonymous596,
-  Anonymous641,
-  Anonymous554,
-  ApplicationReputation,
-  Anonymous577,
-  Anonymous664,
-  Anonymous209,
-  Anonymous438,
-  Anonymous571,
-  Anonymous382,
-  Anonymous455,
-  Anonymous623,
-  CategoryManager,
-  Anonymous503,
-  Anonymous153,
-  Anonymous392,
-  Anonymous567,
-  Anonymous468,
-  UrlClassifierDB,
-  StringBundle,
-  Anonymous211,
-  Anonymous172,
-  Anonymous675,
-  Anonymous070,
-  Anonymous036,
-  Anonymous152,
-  Anonymous463,
-  Anonymous378,
-  Anonymous217,
-  Anonymous637,
-  HttpHandler,
-  Anonymous263,
-  Anonymous443,
-  Anonymous389,
-  Anonymous194,
-  Anonymous053,
-  Anonymous114,
-  Anonymous186,
-  Anonymous504,
-  Anonymous612,
-  Anonymous267,
-  Anonymous021,
-  Anonymous236,
-  Anonymous593,
-  Anonymous009,
-  Anonymous115,
-  OutgoingServer,
-  Anonymous159,
-  Anonymous276,
-  Anonymous113,
-  Anonymous317,
-  Anonymous298,
-  Anonymous175,
-  Anonymous108,
-  Anonymous056,
-  Anonymous161,
-  Anonymous422,
-  Anonymous460,
-  Anonymous105,
-  Anonymous325,
-  Anonymous513,
-  UrlClassifierUtils,
-  Anonymous007,
-  Anonymous561,
-  Anonymous485,
-  Anonymous244,
-  Anonymous140,
-  Anonymous649,
-  Anonymous548,
-  Anonymous352,
-  Anonymous262,
-  Anonymous511,
-  Anonymous020,
-  Anonymous078,
-  Anonymous679,
-  Anonymous350,
-  IdentityCredentialStorageService,
-  CORSPreflightCache,
-  Anonymous657,
-  Anonymous246,
-  Anonymous181,
-  Anonymous425,
-  Anonymous307,
-  Anonymous445,
-  Anonymous250,
-  Anonymous074,
-  Anonymous532,
-  Anonymous635,
-  Anonymous224,
-  AsyncShutdown,
-  Anonymous195,
-  AccountManager,
-  Anonymous145,
-  Anonymous079,
-  Anonymous363,
-  Anonymous219,
-  Anonymous674,
-  Anonymous502,
-  Anonymous609,
-  Anonymous205,
-  Anonymous402,
-  Anonymous633,
-  Anonymous083,
-  Anonymous271,
-  TypeAheadFind,
-  Anonymous570,
-  Anonymous497,
-  Anonymous019,
-  Anonymous041,
-  IdentityCredentialPromptService,
-  Anonymous376,
-  Anonymous284,
-  Anonymous492,
-  Anonymous110,
-  Anonymous495,
-  Anonymous351,
-  Anonymous347,
-  Anonymous310,
-  Anonymous668,
-  Anonymous430,
-  Anonymous669,
-  Anonymous279,
-  Anonymous102,
-  Anonymous017,
-  Anonymous390,
-  Anonymous343,
-  AlertNotification,
-  Anonymous046,
-  nsIContentAnalysis,
-  UrlClassifierPrefixSet,
-  Anonymous003,
-  CookieBannerService,
-  Anonymous134,
-  Anonymous089,
-  Anonymous673,
-  Anonymous296,
-  Anonymous121,
-  Anonymous518,
-  Anonymous660,
-  Anonymous496,
-  Anonymous235,
-  Anonymous104,
-  Anonymous191,
-  Anonymous033,
-  Anonymous583,
-  Anonymous578,
-  Anonymous280,
-  Anonymous491,
-  Anonymous177,
-  Anonymous442,
-  Anonymous179,
-  Anonymous182,
-  Anonymous399,
-  Anonymous671,
-  Anonymous176,
-  SystemInfo,
-  Anonymous519,
-  Anonymous064,
-  Anonymous065,
-  Anonymous437,
-  Anonymous013,
-  FindService,
-  Anonymous447,
-  Anonymous638,
-  Anonymous168,
-  Anonymous309,
-  Anonymous073,
-  Anonymous427,
-  Anonymous470,
-  Anonymous648,
-  Anonymous252,
-  Anonymous109,
-  Anonymous135,
-  Anonymous136,
-  Anonymous682,
-  Anonymous434,
-  Anonymous403,
-  Anonymous426,
-  Anonymous677,
-  Anonymous069,
-  Anonymous412,
-  Anonymous653,
-  Anonymous433,
-  Anonymous630,
-  Anonymous479,
-  DocLoader,
-  Anonymous251,
-  Anonymous208,
-  Anonymous349,
-  Anonymous560,
-  Anonymous097,
-  Anonymous428,
-  Anonymous184,
-  Anonymous335,
-  Anonymous119,
-  Anonymous435,
-  DB,
-  Anonymous544,
-  NSSComponent,
-  History,
-  Anonymous458,
-  Anonymous030,
-  Anonymous275,
-  Anonymous377,
   Anonymous465,
-  Anonymous620,
-  Anonymous542,
-  Anonymous333,
-  Anonymous391,
-  Anonymous288,
-  SiteSecurity,
-  Anonymous253,
-  PrivateAttribution,
-  Anonymous131,
-  Anonymous600,
-  Anonymous663,
-  Anonymous151,
-  Anonymous461,
-  Anonymous094,
-  Anonymous185,
-  Anonymous607,
-  Anonymous587,
-  Anonymous010,
-  Anonymous632,
-  Anonymous541,
-  Anonymous318,
-  Anonymous118,
-  NSSCertificateDB,
-  Anonymous646,
-  Anonymous067,
-  Anonymous303,
-  Anonymous615,
-  HeaderParser,
-  Anonymous077,
-  Anonymous555,
-  Anonymous398,
-  Anonymous266,
-  Anonymous364,
-  Anonymous227,
-  Anonymous540,
-  Anonymous098,
-  Anonymous678,
-  Anonymous405,
-  ThirdPartyUtil,
-  Anonymous441,
-  Anonymous242,
-  Anonymous636,
-  Anonymous197,
-  Anonymous345,
-  Anonymous483,
-  Anonymous154,
-  Anonymous319,
-  Anonymous355,
-  Anonymous120,
-  Anonymous337,
-  Anonymous314,
-  Filter,
-  Anonymous523,
-  Anonymous329,
-  SocketTransport,
-  Anonymous196,
-  PageThumbsStorage,
-  Anonymous269,
-  Anonymous112,
-  Anonymous124,
-  Anonymous652,
-  Anonymous408,
-  Anonymous576,
-  Anonymous088,
-  Anonymous616,
-  Anonymous259,
-  Anonymous383,
-  Anonymous273,
-  Anonymous680,
-  Anonymous301,
-  Anonymous564,
-  Anonymous254,
-  Anonymous466,
-  Anonymous552,
-  Anonymous451,
-  DBView,
-  Anonymous656,
-  Anonymous248,
-  Anonymous385,
-  Anonymous111,
-  Anonymous187,
-  Anonymous580,
-  Anonymous139,
-  Anonymous353,
-  Anonymous060,
-  Anonymous534,
-  Anonymous473,
-  Anonymous499,
-  Anonymous621,
-  Anonymous476,
-  Anonymous099,
-  Anonymous346,
-  Anonymous559,
-  Anonymous539,
-  Anonymous201,
-  FolderNotification,
-  Anonymous008,
-  Anonymous647,
-  Anonymous568,
-  DNS,
-  Anonymous305,
-  CacheStorage,
-  Anonymous546,
-  Anonymous531,
-  Alerts,
-  EffectiveTLD,
-  Anonymous547,
-  Anonymous588,
-  Anonymous050,
-  Anonymous166,
-  Anonymous457,
-  Anonymous322,
-  Anonymous015,
-  Anonymous006,
-  Anonymous203,
-  Anonymous289,
-  Anonymous401,
-  Anonymous093,
-  Anonymous487,
-  Anonymous018,
-  Anonymous644,
-  Anonymous137,
-  Anonymous528,
-  ContentSecurityManager,
-  Anonymous225,
-  Anonymous640,
-  Anonymous627,
-  Anonymous456,
-  Anonymous510,
-  Anonymous106,
-  MimeConverter,
-  Anonymous597,
-  Anonymous416,
-  Anonymous138,
-  WifiMonitor,
-  Anonymous290,
-  Anonymous231,
-  Anonymous341,
-  Anonymous340,
-  Anonymous125,
-  Anonymous147,
-  Anonymous243,
-  Anonymous361,
-  Anonymous212,
-  Anonymous613,
-  Anonymous141,
-  Anonymous454,
-  Anonymous283,
-  Anonymous230,
-  Anonymous044,
-  Anonymous295,
-  URLQueryStringStripper,
-  Anonymous452,
-  Anonymous055,
-  Anonymous475,
-  Anonymous126,
-  Anonymous371,
-  Anonymous011,
-  Anonymous494,
-  Anonymous117,
-  Anonymous400,
-  Anonymous249,
-  Anonymous507,
-  HttpActivityDistributor,
-  Anonymous365,
-  Anonymous066,
-  Anonymous464,
-  Anonymous058,
-  Anonymous617,
-  Anonymous095,
-  Anonymous573,
-  Anonymous162,
-  Anonymous444,
-  Anonymous436,
-  Anonymous143,
-  Anonymous388,
-  Anonymous299,
-  Anonymous414,
-  Anonymous603,
-  Anonymous667,
-  Anonymous658,
-  Anonymous051,
-  Anonymous446,
-  Anonymous129,
-  Anonymous666,
-  Anonymous085,
-  Anonymous642,
-  MailSession,
-  Anonymous477,
-  Anonymous081,
-  Anonymous384,
-  Anonymous654,
-  Anonymous579,
-  Anonymous000,
-  Anonymous221,
-  URILoader,
-  Anonymous411,
-  Anonymous198,
-  Anonymous524,
-  Anonymous149,
-  DownloadPlatform,
-  Anonymous024,
-  Anonymous622,
-  Anonymous155,
-  Anonymous264,
-  Anonymous397,
-  Anonymous575,
-  TextToSubURI,
-  Anonymous190,
-  Anonymous218,
-  Anonymous239,
-  Anonymous598,
-  Anonymous103,
-  ExternalProtocolHandler,
-  Anonymous516,
-  Anonymous202,
-  Anonymous651,
-  Preferences,
-  Anonymous223,
-  NetworkErrorLogging,
-  Anonymous386,
-  Anonymous558,
-  Compose,
-  Anonymous628,
-  Anonymous618,
-  Anonymous038,
-  Anonymous012,
-  Anonymous315,
-  Anonymous650,
-  Anonymous234,
-  Anonymous490,
-  Anonymous287,
-  Anonymous626,
-  Anonymous204,
-  Anonymous486,
-  Anonymous366,
-  Anonymous449,
-  Anonymous180,
-  Anonymous043,
-  Anonymous174,
-  Anonymous080,
-  Anonymous002,
-  Observer,
-  Anonymous379,
-  Anonymous429,
-  Anonymous338,
-  Imap,
-  Anonymous091,
-  Anonymous183,
-  AbManager,
-  Anonymous332,
-  Anonymous599,
-  Anonymous493,
-  Anonymous026,
-  Anonymous467,
-  Anonymous501,
-  Anonymous557,
-  Anonymous514,
-  Anonymous233,
-  Anonymous369,
-  Anonymous193,
-  Tag,
-  Anonymous563,
-  Anonymous595,
-  Anonymous639,
-  Anonymous130,
-  Anonymous257,
-  Anonymous681,
-  Anonymous574,
-  Anonymous553,
-  Anonymous358,
-  Anonymous440,
-  Anonymous356,
-  Anonymous158,
-  Anonymous407,
-  Anonymous040,
-  Anonymous057,
-  Anonymous624,
-  Anonymous545,
-  UUIDGenerator,
-  Anonymous189,
-  Anonymous148,
-  Anonymous472,
-  Dashboard,
+  Anonymous520,
   Anonymous459,
-  Anonymous281,
-  Anonymous591,
-  Anonymous543,
-  Anonymous039,
-  Anonymous116,
-  ExtensionPolicy,
-  Anonymous336,
-  Anonymous200,
-  Anonymous045,
-  Anonymous631,
-  Anonymous144,
-  Anonymous643,
-  NSSErrors,
-  Anonymous565,
-  Anonymous210,
-  Anonymous482,
+  Anonymous636,
+  OutgoingServer,
+  Anonymous380,
+  Anonymous471,
+  Anonymous572,
+  WifiMonitor,
+  Anonymous292,
+  Anonymous067,
+  Anonymous074,
+  Anonymous188,
+  Anonymous279,
+  Anonymous414,
+  Anonymous375,
+  Anonymous179,
+  Anonymous525,
+  Anonymous085,
+  Anonymous489,
+  Anonymous208,
+  Anonymous613,
+  Anonymous091,
+  Anonymous035,
+  URLQueryStringStripper,
+  Anonymous290,
+  Anonymous615,
+  Anonymous060,
+  Anonymous037,
+  Anonymous391,
+  Anonymous016,
+  Anonymous360,
+  Anonymous389,
+  Anonymous093,
+  Anonymous204,
+  AppStartup,
+  Anonymous107,
+  Anonymous532,
+  Anonymous602,
+  Anonymous229,
+  Anonymous614,
+  Anonymous401,
+  ProtocolProxy,
+  Anonymous486,
   Anonymous169,
+  Anonymous406,
+  DBusHandlerApp,
+  Anonymous662,
+  Anonymous205,
+  Anonymous078,
+  Anonymous529,
+  Directory,
+  Anonymous167,
+  Anonymous427,
+  Anonymous611,
+  Anonymous027,
+  Anonymous652,
+  Anonymous231,
+  Anonymous495,
+  Anonymous443,
+  Anonymous334,
+  Anonymous424,
+  Anonymous299,
+  Anonymous001,
+  Anonymous152,
+  Anonymous599,
+  Anonymous160,
+  Anonymous539,
+  Anonymous675,
+  Anonymous617,
+  Anonymous012,
+  NetworkErrorLogging,
+  Anonymous578,
+  Anonymous321,
+  Anonymous618,
+  Anonymous434,
+  Anonymous436,
+  Anonymous581,
+  Anonymous403,
+  Anonymous097,
+  Anonymous114,
+  Anonymous138,
+  Anonymous648,
+  StreamConverter,
+  Anonymous593,
+  Anonymous524,
+  Anonymous531,
+  Anonymous042,
+  Anonymous237,
+  Anonymous146,
+  Anonymous311,
+  Anonymous498,
+  Anonymous381,
+  Anonymous333,
+  Anonymous130,
+  Anonymous560,
+  Anonymous654,
+  Anonymous350,
+  Anonymous353,
+  AccountManager,
+  DBView,
+  Anonymous537,
+  Anonymous473,
+  Anonymous105,
+  Anonymous362,
+  Anonymous171,
+  Anonymous364,
+  Anonymous435,
+  Anonymous151,
+  Anonymous361,
+  Anonymous243,
+  Anonymous284,
+  Anonymous123,
+  Anonymous339,
+  Anonymous166,
+  Anonymous180,
+  Anonymous367,
+  Anonymous607,
+  Anonymous186,
+  Anonymous355,
+  PageThumbsStorage,
+  Anonymous197,
+  Anonymous365,
+  Anonymous466,
+  Anonymous175,
+  Anonymous088,
+  Anonymous374,
+  Anonymous191,
+  Anonymous409,
+  Anonymous075,
+  Anonymous567,
+  Anonymous642,
+  Anonymous491,
+  Anonymous280,
+  HttpActivityDistributor,
+  Anonymous234,
+  Anonymous337,
+  Anonymous548,
+  Anonymous676,
+  Anonymous445,
+  Anonymous351,
+  Anonymous422,
+  AsyncShutdown,
+  Anonymous076,
+  Anonymous073,
+  Anonymous425,
+  Anonymous004,
+  Anonymous565,
+  Anonymous551,
+  Anonymous349,
+  Anonymous543,
+  Anonymous319,
+  Anonymous431,
+  Anonymous399,
+  Compose,
+  Anonymous029,
+  Anonymous683,
+  Anonymous573,
+  Anonymous194,
+  Anonymous044,
+  Anonymous354,
+  Anonymous230,
+  Anonymous006,
+  Anonymous639,
+  Anonymous024,
+  Anonymous270,
+  Anonymous420,
+  Anonymous316,
+  Anonymous065,
+  Anonymous262,
+  Anonymous482,
+  Anonymous104,
+  Anonymous494,
+  Anonymous161,
+  Anonymous182,
+  Anonymous511,
+  Anonymous324,
+  Anonymous009,
+  Anonymous416,
+  Observer,
+  Anonymous153,
+  Anonymous145,
+  Anonymous464,
+  Anonymous390,
+  Anonymous296,
+  Anonymous513,
+  Anonymous410,
+  Anonymous196,
+  Anonymous561,
+  Anonymous327,
+  Anonymous527,
+  Anonymous285,
+  XULRuntime,
+  Anonymous200,
+  Anonymous492,
+  Anonymous000,
+  Anonymous118,
+  Anonymous148,
+  Anonymous412,
+  UUIDGenerator,
+  TypeAheadFind,
+  Anonymous623,
+  Anonymous170,
+  Anonymous553,
+  Anonymous150,
+  Anonymous256,
+  Anonymous598,
+  AuthPromptAdapter,
+  Anonymous382,
+  Anonymous063,
+  Anonymous682,
+  Anonymous545,
+  Anonymous577,
+  Anonymous447,
+  Anonymous444,
+  Anonymous238,
+  Anonymous671,
+  Anonymous089,
+  Anonymous415,
+  DNS,
+  Anonymous592,
+  BounceTrackingProtection,
+  Anonymous385,
+  Anonymous017,
+  Anonymous608,
+  Anonymous069,
+  Anonymous348,
+  RandomGenerator,
+  Anonymous132,
+  Anonymous228,
+  FileProtocolHandler,
+  ServiceWorkerManager,
+  Anonymous379,
+  Anonymous032,
+  SocketTransport,
+  Anonymous215,
+  Anonymous133,
+  Anonymous588,
+  Anonymous501,
+  Anonymous178,
+  Anonymous101,
+  Anonymous014,
+  Anonymous271,
+  Anonymous376,
+  Anonymous055,
+  Anonymous502,
+  AbManager,
+  Anonymous358,
+  Anonymous283,
+  CookieBannerService,
+  Anonymous373,
+  CaptivePortal,
+  Anonymous347,
+  Anonymous314,
+  Anonymous478,
+  Anonymous404,
+  Anonymous220,
+  Anonymous045,
+  Anonymous053,
+  Anonymous535,
+  Anonymous657,
+  Anonymous211,
+  Anonymous628,
+  ScriptSecurityManager,
+  Anonymous213,
+  Anonymous300,
+  Anonymous304,
+  Anonymous221,
   Anonymous255,
+  Anonymous430,
+  Anonymous312,
+  Anonymous481,
+  Anonymous595,
+  Anonymous428,
+  Anonymous542,
+  Anonymous025,
+  Anonymous655,
+  Anonymous252,
+  Anonymous519,
+  Anonymous393,
+  Anonymous040,
+  MimeHeaderParam,
+  Anonymous407,
+  Anonymous232,
+  Anonymous119,
+  Anonymous023,
+  Anonymous173,
+  Anonymous442,
+  Anonymous660,
+  Anonymous589,
+  Anonymous632,
+  Anonymous621,
+  Anonymous413,
+  Anonymous684,
+  Anonymous397,
+  CacheStorage,
+  Anonymous106,
+  Anonymous483,
+  Anonymous202,
+  Anonymous092,
+  Anonymous115,
+  Anonymous363,
+  Anonymous157,
+  Anonymous575,
+  Anonymous139,
+  Anonymous276,
+  Anonymous509,
+  Anonymous174,
+  Anonymous261,
+  Anonymous521,
+  Anonymous522,
+  Locale,
+  Anonymous587,
+  Anonymous506,
+  Anonymous187,
+  Anonymous226,
+  Anonymous164,
+  Anonymous480,
+  Anonymous247,
+  Anonymous651,
+  Anonymous028,
+  Anonymous453,
+  Anonymous108,
+  Anonymous168,
+  Anonymous446,
+  CredentialChooserService,
+  Anonymous259,
+  PrivateAttribution,
+  Anonymous079,
+  Anonymous172,
+  Anonymous441,
+  Anonymous359,
+  Anonymous303,
+  Anonymous307,
+  Anonymous518,
+  Anonymous488,
+  Anonymous342,
+  Anonymous510,
+  Anonymous121,
+  Anonymous003,
+  Anonymous142,
+  UrlClassifierUtils,
+  Anonymous530,
+  PlacesPreviewsHelper,
+  Anonymous672,
+  Anonymous336,
+  Anonymous224,
+  Anonymous562,
+  Anonymous635,
+  Anonymous335,
+  FindService,
+  Anonymous052,
+  Anonymous048,
+  Anonymous046,
+  Anonymous552,
+  Anonymous308,
+  Anonymous646,
+  Anonymous219,
+  Anonymous239,
+  Anonymous137,
+  Anonymous601,
+  Anonymous214,
+  Anonymous026,
+  Anonymous583,
+  Anonymous658,
+  Anonymous246,
+  Anonymous156,
+  Anonymous378,
+  NSSCertificateDB,
+  Anonymous500,
+  Anonymous423,
+  Anonymous640,
+  Anonymous162,
+  Anonymous523,
+  Anonymous512,
+  Anonymous666,
+  Anonymous086,
+  Anonymous038,
+  Anonymous041,
+  Anonymous596,
+  Anonymous120,
+  SiteSecurity,
+  Anonymous630,
+  Anonymous129,
+  Anonymous356,
+  Anonymous110,
+  Anonymous366,
+  GfxInfo,
+  Anonymous072,
+  Anonymous579,
+  Anonymous463,
+  ExtensionPolicy,
+  Anonymous533,
+  TextToSubURI,
+  Anonymous474,
+  Anonymous456,
+  Anonymous627,
+  DocLoader,
+  Anonymous570,
+  Anonymous254,
+  Anonymous638,
+  ThirdPartyUtil,
+  Anonymous544,
+  Anonymous650,
+  Anonymous329,
+  Anonymous504,
+  Anonymous345,
+  Anonymous626,
+  Anonymous128,
+  Anonymous141,
+  Anonymous302,
+  Anonymous606,
+  Anonymous154,
+  Anonymous457,
+  Anonymous619,
+  Anonymous383,
+  Anonymous212,
+  Anonymous134,
+  Anonymous260,
+  Anonymous426,
+  Anonymous020,
+  Anonymous659,
+  Anonymous600,
+  Anonymous569,
+  ApplicationReputation,
+  Anonymous051,
+  Anonymous439,
+  Anonymous507,
+  DB,
+  Anonymous193,
+  Anonymous475,
+  Anonymous438,
+  Anonymous377,
+  Anonymous018,
+  Anonymous536,
+  Anonymous066,
+  AddonContentPolicy,
+  Anonymous183,
+  Anonymous136,
+  Anonymous448,
+  Anonymous460,
+  Anonymous649,
+  Anonymous571,
+  SystemInfo,
+  Anonymous674,
+  Anonymous330,
+  Anonymous112,
+  Anonymous265,
+  Anonymous288,
+  Anonymous011,
+  Anonymous022,
+  Anonymous594,
+  Anonymous176,
+  Anonymous306,
+  DownloadPlatform,
+  Anonymous622,
+  EffectiveTLD,
+  Anonymous043,
+  Anonymous679,
+  Anonymous264,
+  UrlClassifierStreamUpdater,
+  Anonymous203,
+  Anonymous125,
+  Anonymous479,
+  Anonymous371,
+  Anonymous251,
+  Anonymous487,
+  StringBundle,
+  Anonymous056,
+  Anonymous526,
+  Anonymous135,
+  AlertNotification,
+  Anonymous661,
+  Anonymous580,
+  Anonymous090,
+  Anonymous352,
+  Anonymous515,
+  Anonymous616,
+  Anonymous673,
+  Anonymous685,
+  Dashboard,
+  Anonymous604,
+  Anonymous612,
+  Anonymous557,
+  Anonymous077,
+  Anonymous450,
+  Imap,
+  Anonymous163,
+  Anonymous094,
+  Anonymous558,
+  Anonymous033,
+  Anonymous437,
+  Anonymous096,
+  Prefetch,
+  Anonymous122,
+  Anonymous301,
+  URIFixup,
+  Anonymous574,
+  Anonymous209,
+  Anonymous664,
+  Anonymous318,
+  RFPService,
+  Anonymous080,
+  Anonymous653,
+  Anonymous392,
+  Anonymous010,
+  Anonymous467,
+  Anonymous257,
+  Anonymous605,
+  Anonymous631,
+  Anonymous320,
+  Anonymous082,
+  Anonymous242,
+  Anonymous677,
+  Anonymous669,
+  Anonymous068,
+  Anonymous332,
+  Anonymous503,
+  Anonymous665,
+  Anonymous181,
+  Anonymous147,
+  Anonymous643,
+  Anonymous398,
+  URILoader,
+  Anonymous408,
+  Anonymous370,
+  Anonymous064,
+  Anonymous516,
+  Anonymous534,
+  Anonymous461,
+  Anonymous039,
+  Anonymous100,
+  Anonymous455,
+  Anonymous625,
+  NSSErrors,
+  Anonymous549,
+  Anonymous241,
+  Anonymous609,
+  CategoryManager,
+  Anonymous472,
+  Predictor,
+  Anonymous485,
+  Anonymous341,
+  IO,
+  Anonymous517,
+  Anonymous015,
+  Anonymous210,
+  UrlClassifierTestUtils,
+  Anonymous315,
+  Anonymous418,
+  Anonymous131,
+  Anonymous177,
+  Anonymous331,
+  Anonymous637,
+  NSSComponent,
+  Anonymous681,
+  Anonymous223,
+  Anonymous297,
+  Anonymous189,
+  Anonymous250,
+  Alerts,
+  ExternalProtocolHandler,
+  Anonymous620,
+  Anonymous499,
+  Anonymous550,
+  ChromeRegistry,
+  Anonymous083,
+  Folder,
+  Anonymous117,
+  Anonymous233,
+  Anonymous340,
+  Anonymous102,
+  Anonymous113,
+  Anonymous062,
+  ContentSecurityManager,
+  Anonymous591,
+  Anonymous538,
+  Anonymous338,
+  Anonymous484,
+  Anonymous493,
+  Anonymous566,
+  Anonymous031,
+  Anonymous216,
+  Anonymous158,
+  Anonymous663,
+  Anonymous287,
+  Anonymous050,
+  Anonymous629,
+  Anonymous317,
+  Anonymous054,
+  Anonymous528,
+  Anonymous540,
+  Anonymous199,
+  Anonymous034,
+  Anonymous395,
+  Anonymous576,
+  Anonymous490,
+  Anonymous346,
+  Anonymous309,
+  Anonymous387,
+  Anonymous277,
+  Anonymous584,
+  Anonymous111,
+  Anonymous384,
+  Anonymous396,
+  Anonymous293,
+  Anonymous477,
+  Anonymous678,
+  Anonymous198,
+  Anonymous184,
+  Anonymous236,
+  Anonymous273,
+  Anonymous218,
+  Anonymous057,
+  Anonymous468,
+  Anonymous469,
+  Anonymous634,
+  Anonymous305,
+  Anonymous313,
+  Anonymous103,
+  Anonymous546,
+  Anonymous514,
+  Anonymous059,
+  WindowWatcher,
+  Anonymous047,
+  Anonymous369,
+  Anonymous267,
+  Copy,
+  Anonymous061,
+  Anonymous680,
+  Anonymous325,
+  Anonymous323,
+  Anonymous278,
+  Anonymous597,
+  Anonymous155,
+  Anonymous201,
+  Preferences,
+  Anonymous624,
+  Anonymous610,
+  Anonymous248,
+  Anonymous388,
+  Anonymous098,
+  Anonymous670,
+  IDN,
+  Anonymous192,
+  MimeConverter,
+  HttpHandler,
+  Anonymous206,
+  Anonymous326,
+  Anonymous185,
+  Anonymous030,
+  Anonymous195,
+  Anonymous564,
+  Anonymous127,
+  CORSPreflightCache,
+  Anonymous458,
+  Anonymous126,
+  Anonymous462,
+  Anonymous496,
+  nsIContentAnalysis,
+  Anonymous310,
+  MailSession,
+  Anonymous099,
+  Anonymous165,
+  Anonymous586,
+  Anonymous668,
+  Anonymous644,
+  Anonymous402,
+  Anonymous058,
+  Anonymous008,
+  Anonymous476,
+  Anonymous433,
+  Anonymous449,
+  Anonymous286,
+  Anonymous322,
+  Anonymous563,
+  Anonymous249,
+  Anonymous240,
+  History,
+  Anonymous440,
+  Anonymous282,
+  Anonymous451,
+  Anonymous547,
+  Anonymous291,
+  Anonymous190,
+  Tag,
+  Console,
+  Anonymous084,
+  Anonymous144,
+  Anonymous372,
+  Anonymous368,
+  StreamTransport,
+  Anonymous298,
+  Anonymous432,
+  Anonymous274,
+  Anonymous421,
+  Anonymous268,
+  Anonymous281,
+  Anonymous417,
+  Anonymous049,
+  Anonymous582,
+  Anonymous159,
+  Anonymous541,
+  Anonymous235,
+  Anonymous149,
+  Anonymous419,
+  Anonymous585,
+  Anonymous343,
+  Anonymous497,
+  IdentityCredentialStorageService,
+  Anonymous071,
+  Anonymous070,
+  Anonymous007,
+  Anonymous411,
+  Anonymous295,
+  Anonymous087,
+  Anonymous013,
+  Anonymous021,
+  Anonymous095,
+  Anonymous225,
+  FolderNotification,
+  Anonymous275,
+  Anonymous400,
+  Anonymous405,
+  Anonymous559,
+  Anonymous344,
+  Anonymous143,
+  Anonymous294,
+  Anonymous454,
+  Anonymous222,
+  Anonymous633,
+  Anonymous019,
+  Anonymous556,
+  Anonymous005,
+  Anonymous207,
+  Anonymous253,
+  Anonymous554,
+  Anonymous645,
+  Anonymous081,
+  Anonymous603,
+  Anonymous568,
+  Anonymous140,
+  Anonymous266,
+  UrlClassifierPrefixSet,
+  Anonymous641,
+  Anonymous590,
+  Anonymous386,
+  Anonymous429,
+  Anonymous263,
+  Anonymous505,
+  Import,
+  Anonymous002,
+  Anonymous245,
+  Anonymous656,
+  Anonymous289,
+  Anonymous269,
+  UrlClassifierDB,
+  Anonymous508,
+  Anonymous244,
+  Anonymous036,
+  IdentityCredentialPromptService,
+  Anonymous667,
+  Filter,
+  Anonymous258,
+  Anonymous452,
+  PermissionManager,
+  Anonymous116,
+  Anonymous227,
+  Anonymous647,
+  Anonymous124,
+  Anonymous555,
+  Anonymous470,
+  Anonymous328,
+  Anonymous272,
+  Anonymous394,
+  Anonymous217,
+  Anonymous109,
+  Anonymous357,
+  HeaderParser,
 
 }
 
@@ -2523,6 +2525,29 @@ pub mod UrlClassifierStreamUpdater {
     }
 }
 
+#[allow(non_snake_case)]
+pub mod UrlClassifierTestUtils {
+    /// Get the singleton service instance for this component.
+    pub fn service<T: crate::XpCom>() -> Result<crate::RefPtr<T>, nserror::nsresult> {
+        let mut ga = crate::GetterAddrefs::<T>::new();
+        let rv = unsafe { super::Gecko_GetServiceByModuleID(super::ModuleID::UrlClassifierTestUtils, &T::IID, ga.void_ptr()) };
+        if rv.failed() {
+            return Err(rv);
+        }
+        ga.refptr().ok_or(nserror::NS_ERROR_NO_INTERFACE)
+    }
+
+    /// Create a new instance of this component.
+    pub fn create<T: crate::XpCom>() -> Result<crate::RefPtr<T>, nserror::nsresult> {
+        let mut ga = crate::GetterAddrefs::<T>::new();
+        let rv = unsafe { super::Gecko_CreateInstanceByModuleID(super::ModuleID::UrlClassifierTestUtils, &T::IID, ga.void_ptr()) };
+        if rv.failed() {
+            return Err(rv);
+        }
+        ga.refptr().ok_or(nserror::NS_ERROR_NO_INTERFACE)
+    }
+}
+
 #[allow(non_snake_case)]
 pub mod UrlClassifierUtils {
     /// Get the singleton service instance for this component.
